function plotMultilayerSphereTotalFieldUnderPlanewave(varargin)
% Plot the total field of E_r, E_theta, E_phi, H_r, H_theta, H_phi
% evaluated on the Y-Z plane as a plane wave is
% scattered by a three-layer sphere centered at the origin. The plane
% wave propagates in the +z direction. 
%   
% This code takes about 22 min for 44x44 points to finish with a four
% core AMD Phenom(tm) II X4 910 Processor 2600 MHz and about 88 min
% for 88x88 points. Most of the computation is on evaluating the
% bessel functions.
%
%
% Optional:
%
% 'radius'        radius of the sphere (relative to the background
%                 wavelength). Default to [1.5; 1.0; 0.6].
%
% 'sphere'        object of DielectricMaterial. Default to
%                 {DielectricMaterial(2.0, 0.0), ...
%                  DielectricMaterial(8.0,0), ...
%                  DielectricMaterial(4.0,0)}
%
% 'background'    object of DielectricMaterial. Default to
%                 DielectricMaterial(1.0, 0.0).
%
% 'frequency'     scalar to denote the frequency of the plane
%                 wave. Default to 1 Hz.
%
% 'threshold'     threshold of the reciprocal condition number as we invert
%                 the matrix to calculate the coefficients. Default to eps.
%
% 'domain_size'   4x1 vector to specify the size of the domain in
%                 the format  [y_min; y_max; z_min; z_max]
%                 (relative to the background wavelength). Default
%                 to [-0.2; +2; -0.2; +2].
%
% 'N'             2x1 vector to specify the number points to
%                 evaluate the field along the Y and Z axis.
%                 Default to [44, 44].
%
% 'workspace_name'  workspace name to store the
%                   data. Default to [].
%
    
    p = inputParser;
    p.addParameter('radius', [1.5; 1.0; 0.6], @isnumeric);
    p.addParameter('sphere', {DielectricMaterial(2.0, 0.0), ...
                        DielectricMaterial(8.0,0), ...
                        DielectricMaterial(4.0,0)}, @iscell);
    p.addParameter('background', DielectricMaterial(1.0, 0.0), @isobject);
    p.addParameter('frequency', 1.0, @isnumeric);
    p.addParameter('threshold', eps, @isnumeric);
    p.addParameter('domain_size', [-0.2; +2; -0.2; +2], @isvector);
    p.addParameter('N', [44; 44], @isvector);
    p.addParameter('workspace_name', 'multilayer_sphere', @ischar);
    p.addParameter('verbose', 0, @(x)x==0||x==1);
    p.parse(varargin{:});
    if ~(length(p.Results.radius) == length(p.Results.sphere))
        error('Length of radius and sphere should agree. ');
    end
    if (p.Results.verbose)
        disp(p.Results);
    end

    wavelength = getWavelength(p.Results.background, ...
                               p.Results.frequency);
    x          = 0.00;
    y          = linspace(p.Results.domain_size(1), p.Results.domain_size(2), p.Results.N(1));
    z          = linspace(p.Results.domain_size(3), p.Results.domain_size(4), p.Results.N(2));
    
    E_r        = zeros([length(x) length(y) length(z)]);
    E_theta    = zeros([length(x) length(y) length(z)]);
    E_phi      = zeros([length(x) length(y) length(z)]);
    H_r        = zeros([length(x) length(y) length(z)]);
    H_theta    = zeros([length(x) length(y) length(z)]);
    H_phi      = zeros([length(x) length(y) length(z)]);

    iX = 1;
    for iY = 1:p.Results.N(1)
        for iZ = 1:p.Results.N(2)
            [E_r(iX,iY,iZ), E_theta(iX,iY,iZ), E_phi(iX,iY,iZ), ...
             H_r(iX,iY,iZ), H_theta(iX,iY,iZ), H_phi(iX,iY,iZ)] ...
                = getMultilayerSphereFieldUnderPlaneWave(p.Results.radius*wavelength, ...
                                                         p.Results.sphere, ...
                                                         p.Results.background, ...
                                                         [x(iX);y(iY);z(iZ)]*wavelength,...
                                                         p.Results.frequency, ...
                                                         'threshold', ...
                                                         p.Results.threshold);
        end
    end
    
    
    for iY = 1:p.Results.N(1)
        for iZ = 1:p.Results.N(2)
            sensor_location = [x(iX); y(iY); z(iZ)];
            if (norm(sensor_location) > p.Results.radius(1)) % outside the sphere
                [E_x, H_y]      = getPlaneWaveUsingCartesianExpansion(p.Results.background, ...
                                                                  sensor_location*wavelength, ...
                                                                  p.Results.frequency);
                
                % vector transformation from the cartesian
                % to spherical coordinate
                [~, theta, phi] = cart2sph(x(iX), y(iY), z(iZ));
                A               = [+sin(theta)*cos(phi) sin(theta)*sin(phi) +cos(theta);
                                   +cos(theta)*cos(phi) cos(theta)*sin(phi) -sin(theta);
                                   -sin(phi)            cos(phi)             0];
                temp              = A*[E_x;0;0];
                E_r(iX,iY,iZ)     = E_r(iX,iY,iZ)    +temp(1);
                E_theta(iX,iY,iZ) = E_theta(iX,iY,iZ)+temp(2);
                E_phi(iX,iY,iZ)   = E_phi(iX,iY,iZ)  +temp(3);
                temp              = A*[0;H_y;0];
                H_r(iX,iY,iZ)     = H_r(iX,iY,iZ)    +temp(1);
                H_theta(iX,iY,iZ) = H_theta(iX,iY,iZ)+temp(2);
                H_phi(iX,iY,iZ)   = H_phi(iX,iY,iZ)  +temp(3);
            end
        end
    end
    
    if (~isempty(p.Results.workspace_name))
        save(p.Results.workspace_name, 'p', 'x', 'y', 'z', ...
             'E_r', 'E_theta', 'E_phi', ...
             'H_r', 'H_theta', 'H_phi');
    end
    
    set(figure, 'color', 'white');
    subplot(231); plotSphere(y, z, E_r, p.Results.radius); title('E_r');
    subplot(232); plotSphere(y, z, E_theta, p.Results.radius); title('E_\theta');
    subplot(233); plotSphere(y, z, E_phi, p.Results.radius); title('E_\phi');
    subplot(234); plotSphere(y, z, H_r, p.Results.radius); title('H_r');
    subplot(235); plotSphere(y, z, H_theta, p.Results.radius); title('H_\theta');
    subplot(236); plotSphere(y, z, H_phi, p.Results.radius); title('H_\phi');
end

function plotSphere(y, z, data, radius)
    imagesc(y, z, abs(squeeze(data))');
    shading('interp');
    colorbar('EastOutside'); 
    hold('on');
    theta=linspace(0,pi/2,64);
    for iR = 1:length(radius)
        plot(radius(iR)*cos(theta), radius(iR)*sin(theta),'--w','linewidth', 1.0)
    end
    axis('equal', 'tight');
    set(gca,'fontname', 'arial', 'fontsize', 16);
    xlabel('y (\lambda_{0})');
    ylabel('z (\lambda_{0})');
end

